@@ -26,12 +26,13 @@ import sys
 
 import pytest
 
+from pytools import Record
+
 
 logger = logging.getLogger(__name__)
 from typing import FrozenSet
 
 
-@pytest.mark.skipif("sys.version_info < (2, 5)")
 def test_memoize_method_clear():
     from pytools import memoize_method
 
@@ -739,6 +740,23 @@ def test_strtobool():
     assert strtobool(None, False) is False
 
 
+def test_to_identifier() -> None:
+    from pytools import to_identifier
+
+    assert to_identifier("_a_123_") == "_a_123_"
+    assert to_identifier("a_123") == "a_123"
+    assert to_identifier("a 123") == "a123"
+    assert to_identifier("123") == "_123"
+    assert to_identifier("_123") == "_123"
+    assert to_identifier("123A") == "_123A"
+    assert to_identifier("") == "_"
+
+    assert not "a 123".isidentifier()
+    assert to_identifier("a 123").isidentifier()
+    assert to_identifier("123").isidentifier()
+    assert to_identifier("").isidentifier()
+
+
 def test_typedump():
     from pytools import typedump
     assert typedump("") == "str"
@@ -784,6 +802,78 @@ def test_unique():
     assert next(unique([]), None) is None
 
 
+# This class must be defined globally to be picklable
+class SimpleRecord(Record):
+    pass
+
+
+def test_record():
+    r = SimpleRecord(c=3, b=2, a=1)
+
+    assert r.a == 1
+    assert r.b == 2
+    assert r.c == 3
+
+    # Fields are sorted alphabetically in records
+    assert str(r) == "SimpleRecord(a=1, b=2, c=3)"
+
+    # Unregistered fields are (silently) ignored for printing
+    r.f = 6
+    assert str(r) == "SimpleRecord(a=1, b=2, c=3)"
+
+    # Registered fields are printed
+    r.register_fields({"d", "e"})
+    assert str(r) == "SimpleRecord(a=1, b=2, c=3)"
+
+    r.d = 4
+    r.e = 5
+    assert str(r) == "SimpleRecord(a=1, b=2, c=3, d=4, e=5)"
+
+    with pytest.raises(AttributeError):
+        r.ff
+
+    # Test pickling
+    import pickle
+    r_pickled = pickle.loads(pickle.dumps(r))
+    assert r == r_pickled
+
+    # }}}
+
+    # {{{ __slots__, __dict__, __weakref__ handling
+
+    class RecordWithEmptySlots(Record):
+        __slots__ = []
+
+    assert hasattr(RecordWithEmptySlots(), "__slots__")
+    assert not hasattr(RecordWithEmptySlots(), "__dict__")
+    assert not hasattr(RecordWithEmptySlots(), "__weakref__")
+
+    class RecordWithUnsetSlots(Record):
+        pass
+
+    assert hasattr(RecordWithUnsetSlots(), "__slots__")
+    assert hasattr(RecordWithUnsetSlots(), "__dict__")
+    assert hasattr(RecordWithUnsetSlots(), "__weakref__")
+
+    from pytools import ImmutableRecord
+
+    class ImmutableRecordWithEmptySlots(ImmutableRecord):
+        __slots__ = []
+
+    assert hasattr(ImmutableRecordWithEmptySlots(), "__slots__")
+    assert hasattr(ImmutableRecordWithEmptySlots(), "__dict__")
+    assert hasattr(ImmutableRecordWithEmptySlots(), "__weakref__")
+
+    class ImmutableRecordWithUnsetSlots(ImmutableRecord):
+        pass
+
+    assert hasattr(ImmutableRecordWithUnsetSlots(), "__slots__")
+    assert hasattr(ImmutableRecordWithUnsetSlots(), "__dict__")
+    assert hasattr(ImmutableRecordWithUnsetSlots(), "__weakref__")
+
+    # }}}
+
+
 if __name__ == "__main__":
     if len(sys.argv) > 1:
         exec(sys.argv[1])
